<?php
/**
* Shopvote
* Modul für Integration von Shopvote
*
* @author    Markus Engel <info@onlineshop-module.de>
* @copyright Copyright (c) 2017, Onlineshop-Module.de
* @license   commercial, see licence.txt
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

if (!defined('DS')) {
    define('DS', DIRECTORY_SEPARATOR);
}

class Shopvote extends Module
{

    // Eigene Modulversion
    public $gc_module_version = '2.0.3';

    public function __construct()
    {
        /*******************************
        * Einstellungen ab hier bearbeiten
        *******************************/

        // Modulname erhält man automatisch vom Klassennamen (LC)
        $this->name = 'shopvote';

        // Konfigurations Variablen (ohne Prefix)
        $this->config = array(
            'ER_ENABLED' => array('value' => 0),
            'RS_ENABLED' => array('value' => 0),
            'ER_CONTENT_JS' => array('value' => ''),
            'ER_CONTENT_HTML' => array('value' => ''),
            'RS_CONTENT_JS' => array('value' => ''),
        );

        // Hooks zu installieren
        $this->hooks = array('displayOrderConfirmation', 'displayFooter');

        // Modul Tab
        $this->tab = 'front_office_features';

        // Modul Version
        $this->version = '2.0.3';

        // Kompatibilität
        $this->ps_version_compliancy = array(
            'min' => '1.7.2.0',
            'max' => '1.7.999.0',
        );

        // Modul Key von PrestaShop Addons
        $this->module_key = 'c94558d41d3f51a6bdbd4fbabce6cd5f';

        // Modulname und BEschreibung
        $this->displayName = $this->l('ShopVote');
        $this->displayName_1 = $this->l('ShopVote');
        $this->displayName_2 = $this->l('Reviews');
        $this->description = $this->l('ShopVote EasyReviews and RatingStars');

        /*******************************
        * Einstellungen bis hier bearbeiten
        *******************************/

        // Prefix für alle Config Einstellungen festlegen aus Modulnamen (UC)
        $this->cp = Tools::strtoupper($this->name).'_';

        // Sondereinstellungen, wenn Modul ein bezahlmodul ist
        if (get_parent_class() == 'PaymentModule') {
            $this->tab = 'payments_gateways';
            $this->need_instance = 1;
            $this->is_payment = 1;

            $this->controllers = array(
                'payment',
            );

            $this->os = (int)$this->getConfig('OS');
        }

        // Standard Author des Moduls
        $this->author = 'Onlineshop-Module';

        // Parent Modul initialisieren
        parent::__construct();

        // Secure Key generieren
        $this->secure_key = Tools::encrypt($this->name);

        // Deinstallations-Sicherheitsmeldung
        $this->confirmUninstall = $this->l('Are you sure you want to delete your details?');

        if ($this->module_key) {
            $this->is_addons = true;
        }
    }

    public function gcDisplayOptionsPre()
    {
        $this->fields_options = array(
            array(
                'title' => $this->l('EasyReviews'),
                'icon' => 'icon-cog',
                'fields' => array(
                    $this->cp.'ER_CONTENT_HMTL' => array(
                        'type' => 'textarea',
                        'title' => $this->l('EasyReviews HTML-Code'),
                        'desc' => $this->l('EasyReviews javascript content provided by ShopVote.'),
                        'cols' => 30,
                        'rows' => 5,
                    ),
                    $this->cp.'ER_CONTENT_JS' => array(
                        'type' => 'textarea',
                        'title' => $this->l('EasyReviews JavaScript-Code'),
                        'desc' => $this->l('EasyReviews javascript content provided by ShopVote.'),
                        'cols' => 30,
                        'rows' => 5,
                    ),
                    $this->cp.'ER_ENABLED' => array(
                        'type' => 'bool',
                        'title' => $this->l('EasyReviews Enable'),
                        'desc' => $this->l('Enable the EasyReviews feature.'),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                    'name' => 'submitSettings',
                ),
            ),
            array(
                'title' => $this->l('RatingStars'),
                'icon' => 'icon-cog',
                'fields' => array(
                    $this->cp.'RS_CONTENT_JS' => array(
                        'type' => 'textarea',
                        'title' => $this->l('RatingStars JavaScript-Code'),
                        'desc' => $this->l('RatingStars javascript content provided by ShopVote.'),
                        'cols' => 30,
                        'rows' => 5,
                    ),
                    $this->cp.'RS_ENABLED' => array(
                        'type' => 'bool',
                        'title' => $this->l('RatingStars Enable'),
                        'desc' => $this->l('Enable the RatingStars feature.'),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                    'name' => 'submitSettings',
                ),
            ),
        );
    }

    public function gcPostProcessPost()
    {
        if (Tools::isSubmit('submitSettings')) {
            if (!$this->setConfig('ER_CONTENT_JS', trim($this->getValue('ER_CONTENT_JS')), true)) {
                $this->bo_errors[] = sprintf($this->l('Could not update config %s'), $this->l('EasyReviews JavaScript-Code'));
            }

            if (!$this->setConfig('ER_CONTENT_HMTL', trim($this->getValue('ER_CONTENT_HMTL')), true)) {
                $this->bo_errors[] = sprintf($this->l('Could not update config %s'), $this->l('EasyReviews HTML-Code'));
            }

            if (!$this->setConfig('RS_CONTENT_JS', trim($this->getValue('RS_CONTENT_JS')), true)) {
                $this->bo_errors[] = sprintf($this->l('Could not update config %s'), $this->l('RatingStars JavaScript-Code'));
            }

            $this->setConfig('ER_ENABLED', (bool)$this->getValue('ER_ENABLED'));
            $this->setConfig('RS_ENABLED', (bool)$this->getValue('RS_ENABLED'));

            if (count($this->bo_errors) <= 0) {
                $this->bo_confirmations[] = $this->l('Settings updated');
            }
        }
    }

    public function gcHookDisplayHeaderPost(&$params)
    {
        if ($this->getConfig('RS_ENABLED')) {
            $rs_js = $this->getConfig('RS_CONTENT_JS');

            if ($rs_js) {
                $params['html'] .= $rs_js;
            }
        }
    }

    public function hookDisplayOrderConfirmation($params)
    {
        if (isset($params['objOrder'])) {
            $order = $params['objOrder'];
        } else {
            $order = $params['order'];
        }

        $customer = new Customer((int)$order->id_customer);

        $html = '';

        if ($this->getConfig('ER_ENABLED') && Validate::isLoadedObject($order) && Validate::isLoadedObject($customer)) {
            $er_js = $this->getConfig('ER_CONTENT_JS');
            $er_html = $this->getConfig('ER_CONTENT_HMTL');

            if ($er_js && $er_html) {
                $html = str_replace(
                    array('CUSTOMERMAIL', 'ORDERNUMBER'),
                    array($customer->email, $order->reference),
                    $er_html
                );
            }
        }

        return $html;
    }

    public function hookDisplayFooter($params)
    {
        if (!$this->isActive()) {
            return '';
        }

        $html = '';

        if ($this->getConfig('ER_ENABLED') && $this->context->controller->php_self == 'order-confirmation') {
            $er_js = $this->getConfig('ER_CONTENT_JS');
            $er_html = $this->getConfig('ER_CONTENT_HMTL');

            if ($er_js && $er_html) {
                $html .= $er_js;
            }
        }

        return $html;
    }

    /***************************************************************************
    *     INIT 2.0.3
    ***************************************************************************/

    // Backoffice Fehler für Settings Seite
    public $bo_errors = array();
    public $bo_warnings = array();
    public $bo_confirmations = array();

    // Kompatibilität
    public $ps_version_compliancy = array(
        'min' => '1.7.0.0',
        'max' => '1.7.999.999',
    );

    // Wird Modul initialisiert in Module::getModulesOnDisk()
    public $need_instance = 0;

    // Standard Modul-Kategorie
    public $tab = 'front_office_features';

    // Bootstrap seit 1.6 Standard an
    public $bootstrap = true;

    // Array von Hooks, die regisitriert werden
    // $this->hook array('hook_1', 'hook_2', ...);
    public $hooks = array();

    // Modul Variablen, die in der Configuration gespeichert werden. Dem
    // CONFIG_KEY wird automatisch das cp (Config Prefix) vorangesetzt (Standard der
    //Modulname)
    public $config = array();

    // Verzeichnis für Backups der Overrides bei der Installation
    public $backup_dir = 'backup';

    // Moduloptionen (HelperOption)
    public $fields_options = array();

    // Admin-Menüpunkte
    public $tabs = array();

    // Prefix für Configuration Variablen
    public $cp = 'GC_';

    // Ist Modul ein Bezahlmodul? Nur Payment Modul.
    public $is_payment = 0;

    // Bestellstatus nach Bestellabschluss. Nur Payment Modul.
    public $os = null;

    // Eintrag in Zahlart-Tabelle. Nur Payment Modul.
    public $currencies = true;
    public $currencies_mode = 'checkbox';

    // E-Mail Variablen beim absenden einer E-Mail beim Statuswechsel. Nur
    // Payment Modul
    public $extra_mail_vars = array();

    // Soll bei der Installation automatisch ein eigener Bestellstatus
    // installiert werden?
    public $install_order_state = true;

    // Flag bei Installation auf true setzen
    private $is_install_in_process = false;

    // Secure Key für Ajax und Cron Aufrufe
    public $secure_key = '';

    // Bei PS Addons registriert?
    public $is_addons = false;

    /***************************************************************************
    *     INSTALL
    ***************************************************************************/

    /**
    * Installiert das Modul
    * @param null
    * @return bool
    */
    public function install()
    {
        $install = true;

        $this->gcHook(__FUNCTION__, 'pre', $install);

        // Flag setzen, damit Modul erkennt, dass es im installationsprozess ist.
        $this->is_install_in_process = true;

        $this->gcTool('checkOverrides');

        // Overrides überprüfen
        $this->bo_errors = $this->_errors;

        // Bei Fehlern abberechen
        if (!empty($this->bo_errors)) {
            return false;
        }

        $this->gcHook(__FUNCTION__, 'pre_parent');

        // parent Modul installieren
        $install &= (bool)parent::install();

        $this->gcHook(__FUNCTION__, 'post_parent');

        // Hooks installieren
        $install &= (bool)$this->registerHooks();
        // Configurations installieren
        $install &= (bool)$this->installConfig();
        // Overrides schreiben
        $install &= (bool)$this->installOverrideTemplates();
        // Datenbank schreiben
        $install &= (bool)$this->installSql();
        // Admin Menüpunkte einrichten
        $install &= (bool)$this->installTabs($this->tabs);
        // Bestellstatus einrichten
        $install &= (bool)$this->installOrderState();

        // Flag zurück setzen
        $this->is_install_in_process = false;

        $this->gcHook(__FUNCTION__, 'post', $install);

        return (bool)$install;
    }

    /**
    * Registriert alle notwendigen Hooks bei der Installation, angegeben durch
    * $this->hooks. Fügt automatisch weitere Hooks hinzu wenn Bezahlmodul.
    * @param null
    * @return bool
    */
    protected function registerHooks()
    {
        $register = true;

        $this->gcHook(__FUNCTION__, 'pre', $register);

        // Modul wird immer im Header registriert um JS und CSS hinzuzufügen
        $this->hooks = array_unique(array_merge($this->hooks, array(
            'displayHeader'
        )));

        // Wenn Bezahlmodul, dann werden die einzelnen Bezahl-Hooks hinzugefügt
        if ($this->is_payment) {
            $this->hooks = array_unique(array_merge($this->hooks, array(
                'paymentOptions',
                'paymentReturn',
            )));
        }

        // Jeden angegebenen Hook hinzufügen
        foreach ($this->hooks as $hook_name) {
            if (method_exists('Hook', 'registerHook')) {
                $register &= Hook::registerHook($this, $hook_name);
            } else {
                $register &= $this->registerHook($hook_name);
            }
        }

        $this->gcHook(__FUNCTION__, 'post', $register);

        return (bool)$register;
    }

    /**
    * Installiert Admin Menüpunkte angegeben durch $this->tabs
    * @param null
    * @return bool
    */
    public function installTabs(array $tabs, $id_parent = null)
    {
        $install = true;

        $this->gcHook(__FUNCTION__, 'pre', $install);

        foreach ($tabs as $value) {
            // Wenn Tab mit der Klasse schon existiert...
            if (Tab::getIdFromClassName($value['class_name'])) {
                // ... Tab Installation abbrechen
                return true;
            }

            // Neuen Tab erstellen
            $tab = new Tab();
            $tab->active = 1;
            $tab->class_name = $value['class_name'];
            $tab->name = array();

            // Standard-Namen setzen, wenn nicht vorhanden
            if (!isset($value['name'])) {
                $value['name'] = $this->displayName;
            }

            foreach (Language::getLanguages(true) as $lang) {
                $tab->name[$lang['id_lang']] = $value['name'];
            }

            // Tab Parent ermitteln
            if ($id_parent !== null) {
                $tab->id_parent = (int)$id_parent;
            } elseif (isset($value['parent']) && is_numeric($value['parent'])) {
                $tab->id_parent = (int)$value['parent'];
            } elseif (isset($value['parent']) && $value['parent']) {
                $tab->id_parent = (int)Tab::getIdFromClassName($value['parent']);
            } else {
                $tab->id_parent = -1;
            }

            $tab->module = (isset($value['module']) && $value['module']) ? $value['module'] : '';

            $install &= (bool)$tab->add();

            // Unter-Tabs registrieren
            if ($install && isset($value['children']) && $value['children']) {
                $install &= (bool)$this->installTabs($value['children'], $tab->id);
            }
        }

        $this->gcHook(__FUNCTION__, 'post', $install);

        return (bool)$install;
    }

    /**
    * Fügt neue Configuration Einträge hinzu, angegeben durch $this->config
    * @param null
    * @return bool
    */
    protected function installConfig()
    {
        $install = true;

        $this->gcHook(__FUNCTION__, 'pre', $install);
        
        if (!$this->getConfigGlobal('LICENSE')) {
            $install &= (bool)$this->setConfigGlobal('LICENSE', '');
            $install &= (bool)$this->setConfigGlobal('PRIVACY', '0');
        }

        foreach ($this->config as $key => $value) {
            if (!isset($value['value'])) {
                continue;
            }

            $values_lang = array();

            if (isset($value['lang']) && $value['lang']) {
                foreach (Language::getLanguages(true) as $lang) {
                    $values_lang[$lang['id_lang']] = $value['value'];
                }
            } else {
                $values_lang = array($value['value']);
            }

            if (isset($value['global']) && $value['global']) {
                $install &= (bool)Configuration::updateGlobalValue($this->cp.$key, $values_lang);
            } else {
                $install &= (bool)$this->setConfig($key, $values_lang);
            }
        }

        $this->gcHook(__FUNCTION__, 'post', $install);

        return (bool)$install;
    }

    /**
    * SQL Datei /sql/install.sql parsen und ausführen
    * @param null
    * @return bool
    */
    public function installSql()
    {
        $install = true;

        $this->gcHook(__FUNCTION__, 'pre', $install);

        $install &= (bool)$this->executeSql($this->local_path.'sql/install.sql');

        $this->gcHook(__FUNCTION__, 'post', $install);

        return (bool)$install;
    }

    /**
    * Admin Templates, die überschrieben werden automatisch in das richtige
    * Verzeichnis kopieren bei der Installation
    * @param null
    * @return bool
    */
    public function installOverrideTemplates()
    {
        $install = true;

        $this->gcHook(__FUNCTION__, 'pre', $install);

        if ($install && is_dir($this->local_path.'override/controllers/admin/templates/')) {
            Tools::recurseCopy(
                $this->local_path.'override/controllers/admin/templates/',
                _PS_OVERRIDE_DIR_.'controllers/admin/templates/'
            );
        }

        $this->gcHook(__FUNCTION__, 'post', $install);

        return $install;
    }

    /**
    * Beim aktivieren sollen die Overrides wieder geschrieben werden, aber nicht
    * bei der Installation
    * @param force_all: aktiviert das Modul für alle Devices
    * @return bool
    */
    public function enable($force_all = false)
    {
        $enable = true;

        $this->gcHook(__FUNCTION__, 'pre', $enable);

        $this->gcHook(__FUNCTION__, 'pre_parent', $enable);

        $enable &= (bool)parent::enable($force_all);

        $this->gcHook(__FUNCTION__, 'post_parent', $enable);

        if (!$this->is_install_in_process) {
            $enable &= (bool)$this->installOverrideTemplates();
        }

        $this->gcHook(__FUNCTION__, 'post', $enable);

        return (bool)$enable;
    }

    /**
    * Bei Bezahlmodul wird automatisch ein Bestellstatus hinzugefügt abhängig
    * von $this->install_order_state
    * @param null
    * @return bool
    */
    public function installOrderState()
    {
        $this->gcHook(__FUNCTION__, 'pre');

        // Wenn kein Bezahlstatus installiert werden soll
        if (!$this->is_payment || !$this->install_order_state) {
            return true;
        }

        // Bestellstatus von vorherigen Installationen ermitteln
        $id_order_state = (int)DB::getInstance()->getValue("
            SELECT `id_order_state` 
            FROM `"._DB_PREFIX_."order_state` 
            WHERE `module_name`='".pSQL($this->name)."'
        ");

        // Wenn vorhanden, dann soll der Vorhandene Bestellstatus verwendet werden und kein neuer installiert werden
        if ($id_order_state) {
            $order_state = new OrderState($id_order_state);

            if (Validate::isLoadedObject($order_state)) {
                $this->setConfig('OS', $id_order_state);

                return true;
            }
        }

        // Neuen Bestellstatus erstellen
        $order_state = new OrderState();

        $order_state->name = array();
        $order_state->template = array();

        foreach (Language::getLanguages() as $language) {
            $order_state->name[$language['id_lang']] = sprintf(
                $this->l('Awaiting payment: %s'),
                $this->displayName
            );

            $order_state->template[$language['id_lang']] = $this->name;
        }

        $order_state->send_email   = false;
        $order_state->module_name  = $this->name;
        $order_state->invoice      = false;
        $order_state->color        = 'RoyalBlue';
        $order_state->unremovable  = false;
        $order_state->logable      = false;
        $order_state->delivery     = false;
        $order_state->hidden       = false;
        $order_state->shipped      = false;
        $order_state->paid         = false;
        $order_state->deleted      = false;
        $order_state->pdf_invoice  = false;
        $order_state->pdf_delivery = false;

        if ($order_state->add()) {
            // Wenn Bestellstatus-Icon vorhanden, kopieren
            if (is_file($this->local_path.'views/img/os.gif')) {
                @copy($this->local_path.'views/img/os.gif', _PS_IMG_DIR_.'os/'.(int)($order_state->id).'.gif');
            }

            // Wenn E-Mail Vorlagen für den Bestellstatus vorhanden, kopieren in _PS_MAIL_DIR_
            $mails = $this->getOrderStatusMails();

            if ($mails) {
                $order_state->send_email = true;
                $order_state->update;
            }

            foreach ($mails as $template => $iso_files) {
                foreach ($iso_files as $iso_file) {
                    @copy($this->local_path.'os_mail/'.$iso_file, _PS_MAIL_DIR_.$iso_file);
                }

                if (Module::isInstalled('advancedeucompliance')) {
                    require_once(_PS_MODULE_DIR_.'advancedeucompliance/entities/AeucEmailEntity.php');

                    $new_email = new AeucEmailEntity();
                    $new_email->filename = $template;
                    $new_email->display_name = $this->displayName;
                    $new_email->save();
                }
            }

            // Bestellstatus und original Bestellstatus festlegen.
            $this->setConfig('OS', $order_state->id);

            return $order_state->id;
        }

        $this->gcHook(__FUNCTION__, 'post');

        return false;
    }

    /***************************************************************************
    *     UNINSTALL
    ***************************************************************************/

    /**
    * Deinstalliert das Modul
    * @param null
    * @return bool
    */
    public function uninstall()
    {
        $uninstall = true;

        $this->gcHook(__FUNCTION__, 'pre', $uninstall);

        $this->is_install_in_process = true;

        $uninstall &= (bool)$this->removeOrderState();
        $uninstall &= (bool)$this->uninstallTabs($this->tabs);
        $uninstall &= (bool)$this->uninstallSql();
        $uninstall &= (bool)$this->uninstallOverrideTemplates();
        $uninstall &= (bool)$this->uninstallConfig();

        $this->gcHook(__FUNCTION__, 'pre_parent', $uninstall);

        $uninstall &= (bool)parent::uninstall();

        $this->gcHook(__FUNCTION__, 'post_parent', $uninstall);

        $this->bo_errors = array_merge($this->_errors, $this->bo_errors);

        $this->is_install_in_process = false;

        $this->gcHook(__FUNCTION__, 'post', $uninstall);

        return (bool)$uninstall;
    }

    /**
    * Deaktiviert das Modul. Override Templates sollen hierfür entfernt werden.
    * @param force_all: Deaktiviert das Modul für alle Devices
    * @return bool
    */
    public function disable($force_all = false)
    {
        $disable = true;

        $this->gcHook(__FUNCTION__, 'pre', $disable);

        $this->gcHook(__FUNCTION__, 'pre_parent', $disable);

        $disable &= (bool)parent::disable($force_all);

        $this->gcHook(__FUNCTION__, 'post_parent', $disable);

        if (!$this->is_install_in_process) {
            $disable &= $this->uninstallOverrideTemplates();
        }

        $this->gcHook(__FUNCTION__, 'post', $disable);

        return (bool)$disable;
    }

    /**
    * Löscht die Admin-Menüpunkte
    * @param null
    * @return bool
    */
    public function uninstallTabs(array $tabs)
    {
        $uninstall = true;

        $this->gcHook(__FUNCTION__, 'pre', $uninstall);

        foreach ($tabs as $value) {
            if ($uninstall && isset($value['children']) && $value['children']) {
                $uninstall &= (bool)$this->uninstallTabs($value['children']);
            }

            if (!$id_tab = (int)Tab::getIdFromClassName($value['class_name'])) {
                return true;
            }

            $tab = new Tab($id_tab);
            $uninstall &= (bool)$tab->delete();
        }

        $this->gcHook(__FUNCTION__, 'post', $uninstall);

        return (bool)$uninstall;
    }

    /**
    * Löscht die Einträge des Moduls aus der Configuration Tabelle. Das
    * Config Präfix wird automatisch vorangestellt
    * @param null
    * @return bool
    */
    protected function uninstallConfig()
    {
        $uninstall = true;

        $this->gcHook(__FUNCTION__, 'pre', $uninstall);

        foreach ($this->config as $key => $value) {
            // PS validator sucks!
            $value = $value;
            $uninstall &= (bool)$this->deleteConfig($key);
        }

        $uninstall &= (bool)$this->deleteConfig('LICENSE');
        $uninstall &= (bool)$this->deleteConfig('PRIVACY');

        $this->gcHook(__FUNCTION__, 'post', $uninstall);

        return (bool)$uninstall;
    }

    /**
    * führt das Script /sql/uninstall.sql aus
    * @param null
    * @return bool
    */
    public function uninstallSql()
    {
        $uninstall = true;

        $this->gcHook(__FUNCTION__, 'pre', $uninstall);

        $uninstall &= (bool)$this->executeSql($this->local_path.'sql/uninstall.sql');

        $this->gcHook(__FUNCTION__, 'post', $uninstall);

        return (bool)$uninstall;
    }

    /**
    * Override Templates deinstallieren
    * @param null
    * @return bool
    */
    public function uninstallOverrideTemplates()
    {
        $uninstall = true;

        $this->gcHook(__FUNCTION__, 'pre', $uninstall);

        if (!is_dir($this->local_path.'override/')) {
            return true;
        }

        $files = Tools::scandir($this->local_path.'override/', 'tpl', '', true);

        foreach ($files as $file) {
            if (is_file(_PS_OVERRIDE_DIR_.$file)) {
                $uninstall &= (bool)unlink(_PS_OVERRIDE_DIR_.$file);
            }
        }

        $this->gcHook(__FUNCTION__, 'post', $uninstall);

        return (bool)$uninstall;
    }

    /**
    * Bestellstatus löschen. Nur bei Bezahlmodul. Wenn Bestellstatus schon
    * aktive Bestellungen hat, wird dieser nicht gelöscht, da die Bestellungen
    * dann einen fehlenden History Eintrag haben
    * @param null
    * @return bool
    */
    public function removeOrderState()
    {
        $this->gcHook(__FUNCTION__, 'pre');

        // Wenn Bestellstatus nicht installiert worden
        if (!$this->is_payment || !$this->install_order_state) {
            return true;
        }

        // Original Bestellstatus ermitteln
        $id_order_state = (int)DB::getInstance()->getValue("
            SELECT `id_order_state` 
            FROM `"._DB_PREFIX_."order_state` 
            WHERE `module_name`='".pSQL($this->name)."'
        ");

        if ($id_order_state) {
            // wurde dieser Bestellstatus in mindestens einer Bestellung verwendet?
            $is_valid = DB::getInstance()->getValue("
                SELECT 1
                FROM `"._DB_PREFIX_."order_history`
                WHERE `id_order_state`='".(int)$id_order_state."'
            ");

            if (!$is_valid) {
                $order_state = new OrderState($id_order_state);

                if (Validate::isLoadedObject($order_state)) {
                    $mails = $this->getOrderStatusMails();

                    foreach ($mails as $template => $iso_files) {
                        foreach ($iso_files as $iso_file) {
                            if (is_file(_PS_MAIL_DIR_.$iso_file)) {
                                unlink(_PS_MAIL_DIR_.$iso_file);
                            }
                        }

                        if (Module::isInstalled('advancedeucompliance')) {
                            $id_mail = (int)DB::getInstance()->getValue("
                                SELECT `id_mail`
                                FROM `"._DB_PREFIX_."aeuc_email`
                                WHERE `filename`='".pSQL($template)."'
                            ");

                            if ($id_mail) {
                                DB::getInstance()->getValue("
                                    DELETE FROM `"._DB_PREFIX_."aeuc_email`
                                    WHERE `id_mail`='".pSQL($id_mail)."'
                                ");

                                DB::getInstance()->getValue("
                                    DELETE FROM `"._DB_PREFIX_."aeuc_cmsrole_email`
                                    WHERE `id_mail`='".pSQL($id_mail)."'
                                ");
                            }
                        }
                    }

                    $order_state->delete();
                }
            }
        }

        $this->deleteConfig('OS');

        $this->gcHook(__FUNCTION__, 'post');

        return true;
    }

    /***************************************************************************
    *     RESET
    ***************************************************************************/

    /**
    * Modul Reset
    * @param null
    * @return bool
    */
    public function reset()
    {
        $reset = true;

        $this->gcHook(__FUNCTION__, 'pre', $reset);

        $reset &= (bool)$this->installConfig();
        $reset &= (bool)$this->resetSql();
        $reset &= (bool)$this->registerHooks();

        $this->gcHook(__FUNCTION__, 'post', $reset);

        return (bool)$reset;
    }

    /**
    * Führt das /sql/reset.sql Script aus
    * @param null
    * @return bool
    */
    public function resetSql()
    {
        $reset = true;

        $this->gcHook(__FUNCTION__, 'pre', $reset);

        $reset &= (bool)$this->executeSql($this->local_path.'sql/reset.sql');

        $this->gcHook(__FUNCTION__, 'post', $reset);

        return (bool)$reset;
    }

    /***************************************************************************
    *     BO Template
    ***************************************************************************/

    /**
    * Zeigt Ansicht für die Modulkonfiguration im Backoffice
    * @param null
    * @return string
    */
    public function getContent()
    {
        $html = '';

        $this->gcHook(__FUNCTION__, 'pre', $html);

        // Admin Javascript und CSS integrieren
        $this->context->controller->addJs(__PS_BASE_URI__.'modules/'.$this->name.'/views/js/admin.js');
        $this->context->controller->addCSS(__PS_BASE_URI__.'modules/'.$this->name.'/views/css/admin.css');

        // abgesendete Daten verarbeiten
        $html .= $this->postProcess();

        // Modulinfos anzeigen
        $html .= $this->displayInfo();

        // Fehler, Hinweise und Erfolgsnachrichten anzeigen
        $html .= $this->displayMessages();

        if (!$this->is_addons
            && !$this->getConfigGlobal('LICENSE')
            && !$this->isDevMode()
        ) {
            // ... Wird Lizenz-Header angezeigt
            $html .= $this->gcTool('displayLicense');
        } else {
            // ... ansonsten Modul-Konfigurationsseite
            $html .= $this->displayOptions();
        }

        $this->gcHook(__FUNCTION__, 'post', $html);

        return $html;
    }

    /**
    * Zeigt Modul Info-Header
    * @param null
    * @return string
    */
    public function displayInfo()
    {
        $html = '';

        $this->gcHook(__FUNCTION__, 'pre', $html);

        $this->smarty->assign(array(
            '_path'             => __PS_BASE_URI__.'modules/'.$this->name.'/',
            'name'              => $this->name,
            'displayName'       => $this->displayName,
            'displayName_1'     => $this->displayName_1,
            'displayName_2'     => $this->displayName_2,
            'author'            => $this->author,
            'description'       => $this->description,
            'token'             => Tools::getAdminTokenLite('AdminModules'),
            'toggleInfo'        => $this->context->cookie->__get('toggleInfo'.$this->name),
            'license_key'       => ($this->isDevMode() ?
                $this->l('Dev Mode!') :
                $this->getConfigGlobal('LICENSE')),
            'version'           => $this->version,
            'gc_module_version' => $this->gc_module_version,
            'l_info_1'          => $this->l('Onlineshop-Module creates professional and individual modules and addons for PrestaShop onlinestores since 2010.'),
            'l_info_2'          => $this->l('Get more information from the links below.'),
            'l_info_3'          => $this->l('Please consider our license disclaimer.'),
            'l_info_4'          => $this->l('License key'),
            'is_addons'         => $this->is_addons,
            'has_ads'           => is_file($this->getLocalPath().'views/templates/admin/info_ads.tpl')
        ));

        $html .= $this->fetch($this->getLocalPath().'views/templates/admin/info.tpl');
        
        $this->gcHook(__FUNCTION__, 'post', $html);

        return $html;
    }

    /**
    * Zeigt Fehlermeldungen des Moduls
    * @param null
    * @return string
    */
    public function displayMessages()
    {
        $html = '';

        $this->gcHook(__FUNCTION__, 'pre', $html);

        if ($this->bo_errors) {
            $html .= $this->displayError(array_unique($this->bo_errors));
        }
        if ($this->bo_warnings) {
            $html .= $this->displayWarning(array_unique($this->bo_warnings));
        }
        if ($this->bo_confirmations) {
            $html .= $this->displayConfirmation(array_unique($this->bo_confirmations));
        }

        $this->gcHook(__FUNCTION__, 'post', $html);

        return $html;
    }

    /**
    * Stellt Fehler als HTML Block zusammen
    * @param array/string
    * @return string
    */
    public function displayError($errors)
    {
        $this->error = true;

        $this->smarty->assign(array(
            'msg' => $errors,
            'class' => 'module_error alert alert-danger'
        ));

        return $this->fetch($this->getLocalPath().'views/templates/admin/message.tpl');
    }

    /**
    * Stellt Warnungen als HTML Block zusammen
    * @param array/string
    * @return string
    */
    public function displayWarning($warnings)
    {
        $this->error = true;

        $this->smarty->assign(array(
            'msg' => $warnings,
            'class' => 'module_warning alert alert-warning'
        ));

        return $this->fetch($this->getLocalPath().'views/templates/admin/message.tpl');
    }

    /**
    * Stellt Erfolgsnachrichten als HTML Block zusammen
    * @param array/string
    * @return string
    */
    public function displayConfirmation($confirmations)
    {
        $this->smarty->assign(array(
            'msg' => $confirmations,
            'class' => 'module_confirmation conf confirm alert alert-success'
        ));

        return $this->fetch($this->getLocalPath().'views/templates/admin/message.tpl');
    }

    /**
    * Verarbeitet Eingaben von der Modul Konfigurationsseite
    * @param null
    * @return null
    */
    public function postProcess()
    {
        $this->gcHook(__FUNCTION__, 'pre');
        
        if (Tools::isSubmit('submitLicense')) {
            $this->gcTool('submitLicense');
        }

        if (Tools::isSubmit('submitToggleInfo')) {
            $this->context->cookie->__set('toggleInfo'.$this->name, (bool)Tools::getValue('submitToggleInfo'));
            echo Tools::jsonEncode($this->context->cookie->__get('toggleInfo'.$this->name));
            die();
        }
        
        if ($this->is_payment && Tools::isSubmit('submitSettings')) {
            if (self::setConfig('OS', (int)$this->getValue('OS'))) {
                $this->bo_confirmations[] = $this->l('Settings updated');
            } else {
                $this->bo_errors[] = $this->l('Could not update Settings');
            }
        }

        $this->gcHook(__FUNCTION__, 'post');
    }

    /**
    * Zeigt Modul Konfigurationsoptionen an
    * @param null
    * @return string
    */
    public function displayOptions()
    {
        $html = '';

        $this->gcHook(__FUNCTION__, 'pre', $html);

        if ($this->is_payment) {
            if (!isset($this->fields_options[0]['fields'][$this->cp.'OS'])) {
                $this->fields_options[0]['fields'][$this->cp.'OS'] = array(
                    'title' => $this->l('Selected Order State'),
                    'type' => 'select',
                    'desc' => $this->l('The order state after customer placed an order.'),
                    'list' => OrderState::getOrderStates($this->context->language->id),
                    'identifier' => 'id_order_state',
                    'name' => 'name',
                );
            }
            if (!isset($this->fields_options[0]['submit'])) {
                $this->fields_options[0]['submit'] = array(
                    'name' => 'submitSettings',
                    'title' => $this->l('Save'),
                );
            }
        }

        $helper = new HelperOptions();

        $helper->id = Tab::getIdFromClassName('AdminModules');
        $helper->default_form_language = $this->context->language->id;
        $helper->show_toolbar = false;
        $helper->module = $this;
        $helper->title = $this->displayName;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;

        $html .= $helper->generateOptions($this->fields_options);

        $this->gcHook(__FUNCTION__, 'post', $html);

        return $html;
    }

    /***************************************************************************
    *     HOOKS
    ***************************************************************************/

    /**
    * Front-Office Header Hook. Integriert Javascripte und CSS auf allen
    * Frontoffice Seiten
    * @param params
    * @return null
    */
    public function hookDisplayHeader($params)
    {
        $params['html'] = '';

        $this->gcHook(__FUNCTION__, 'pre', $params);

        if (!$this->isActive()) {
            return '';
        }

        $this->context->controller->addCSS($this->_path.'views/css/'.$this->name.'.css');
        $this->context->controller->addJS($this->_path.'views/js/'.$this->name.'.js');

        if ($this->is_payment) {
            $this->context->controller->addJS($this->_path.'views/js/pp_checkout.js');
        }
    
        $this->gcHook(__FUNCTION__, 'post', $params);
        
        return $params['html'];
    }

    /**
    * Wenn Bezahlmodul Anzeige der Bezahloption im Bestellprozess
    * @param params
    * @return string
    */
    public function hookDisplayPayment($params)
    {
        $this->gcHook(__FUNCTION__, 'pre', $params);

        if (!$this->isActive() || !$this->is_payment) {
            return '';
        }

        if (isset($params['cancel_hook']) && $params['cancel_hook']) {
            return '';
        }

        $this->smarty->assign(array(
            'this_path'       => $this->_path,
            'this_path_ssl'   => Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/',
        ));
        
        $this->gcHook(__FUNCTION__, 'post', $params);
        
        return $this->display($this->getLocalPath(), 'displayPayment.tpl');
    }

    /**
    * Wenn Bezahlmodul Anzeige der EU Bezahloption im Bestellprozess
    * @param params
    * @return string
    */
    public function hookDisplayPaymentEU($params)
    {
        $this->gcHook(__FUNCTION__, 'pre', $params);
        
        if (!$this->isActive() || !$this->is_payment) {
            return;
        }
        
        if (isset($params['cancel_hook']) && $params['cancel_hook']) {
            return '';
        }

        $this->smarty->assign(array(
            'this_path'       => $this->_path,
            'this_path_ssl'   => Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/',
        ));
        
        $params['payment_options'] = array(
            'cta_text' => $this->displayName,
            'logo'     => Media::getMediaPath($this->getLocalPath().'/views/img/payment.png'),
            'form'     => $this->display($this->getLocalPath(), 'displayPaymentEU.tpl'),
        );

        $this->gcHook(__FUNCTION__, 'post', $params);

        return $params['payment_options'];
    }

    /**
    * Integriert Modul nach Bestellprozess auf Bestellbestätigungsseite
    * @param params
    * @return string
    */
    public function hookDisplayPaymentReturn($params)
    {
        $this->gcHook(__FUNCTION__, 'pre', $params);

        if (!$this->isActive() || !$this->is_payment) {
            return;
        }

        if (isset($params['cancel_hook']) && $params['cancel_hook']) {
            return '';
        }

        if (isset($params['objOrder'])) {
            $order = $params['objOrder'];
        } else {
            $order = $params['order'];
        }

        $currency = new Currency($order->id_currency);

        if ($order->module != $this->name) {
            return;
        }

        $this->context->controller->addCSS($this->_path.'/views/css/'.$this->name.'.css');
        $this->context->controller->addJS($this->_path.'/views/js/'.$this->name.'.js');

        $state = $order->getCurrentState();
        $customer = new Customer($order->id_customer);

        $html = '';

        if (in_array($state, array(
            $this->os,
            Configuration::get('PS_OS_OUTOFSTOCK'),
            Configuration::get('PS_OS_OUTOFSTOCK_UNPAID')
        ))) {
            $this->smarty->assign(array(
                'status'     => 'ok',
                'is_guest'   => $customer->is_guest,
                'email'      => $customer->email,
                'id_order'   => $order->id,
                'reference'  => $order->reference,
                'total_paid' => Tools::displayPrice($order->total_paid, $currency, false),
                'shop_name'  => $this->context->shop->name,
            ));
        } else {
            $this->smarty->assign(array(
                'status' => 'error',
                'id_order' => $order->id
            ));
        }

        $this->smarty->assign(array(
            'this_path'     => $this->_path,
            'this_path_ssl' => Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/'
        ));
        
        $html .= $this->display($this->getLocalPath(), 'displayPaymentReturn.tpl');
        
        $this->gcHook(__FUNCTION__, 'post', $html);
        
        return $html;
    }

    /***************************************************************************
    *     HELPER
    ***************************************************************************/

    /**
    * Tauscht Platzhalter in SQL Datei aus durch PS Variablen: PREFIX_ ENGINE_TYPE COLLATION und führ das SQL Script aus
    * @param string file: Dateiname der auszuführenden Datei
    * @return bool
    */
    public function executeSql($file)
    {
        $execute = true;

        $this->gcHook(__FUNCTION__, 'pre', $execute);

        if (is_file($file)) {
            $allowed_collation = array('utf8_general_ci', 'utf8_unicode_ci', 'utf8_bin');
            $collation_database = Db::getInstance()->getValue('SELECT @@collation_database');

            $search = array('PREFIX_', 'ENGINE_TYPE', 'COLLATION');
            $replace = array(
                _DB_PREFIX_,
                _MYSQL_ENGINE_,
                (
                    empty($collation_database) ||
                    !in_array($collation_database, $allowed_collation)
                ) ? '' : 'COLLATE '.$collation_database
            );

            $sql = Tools::file_get_contents($file);
            $sql = str_replace($search, $replace, $sql);
            $sql = preg_split("/;\s*[\r\n]+/", $sql);

            foreach ($sql as $query) {
                if ($query) {
                    $execute &= (bool)Db::getInstance()->execute(trim($query));
                }
            }
        }

        $this->gcHook(__FUNCTION__, 'post', $execute);

        return (bool)$execute;
    }
    
    /**
    * Ermittelt, ob in DB Tabelle eine Spalte existiert
    * @param
    * @return bool
    */
    public function dbColumnExists($table_name, $column_name)
    {
        $result = DB::getInstance()->executeS("
            SELECT COLUMN_NAME, DATA_TYPE, IS_NULLABLE, COLUMN_DEFAULT
            FROM INFORMATION_SCHEMA.COLUMNS
            WHERE TABLE_NAME = '"._DB_PREFIX_.pSQL($table_name)."'
            AND TABLE_SCHEMA = '"._DB_NAME_."'
        ");

        if (!$result) {
            return null;
        }

        foreach ($result as $row) {
            if ($row['COLUMN_NAME'] == $column_name) {
                return true;
            }
        }

        return false;
    }

    /**
    * Überprüft ob das Modul aktiv ist. U.a. wir auch die Lizenz in die Überprüfung mit eingeschlossen
    * @param
    * @return bool
    */
    public function isActive()
    {
        $active = true;

        $this->gcHook(__FUNCTION__, 'pre', $active);

        $active &= $this->active;

        $this->gcHook(__FUNCTION__, 'post', $active);

        return (bool)$active;
    }

    /**
    * Wenn der Prestashop auf einem localhost ausgeführt wird ist dies eine Testversion
    * @param
    * @return bool
    */
    public function isDevMode()
    {
        $is_devmode = true;

        $this->gcHook(__FUNCTION__, 'pre', $is_devmode);

        $is_devmode &= (
            $_SERVER['SERVER_NAME'] == 'localhost' ||
            $_SERVER['SERVER_ADDR'] == '127.0.0.1' ||
            $_SERVER['SERVER_ADDR'] == '::1'
        );

        $this->gcHook(__FUNCTION__, 'post', $is_devmode);

        return (bool)$is_devmode;
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency((int)($cart->id_currency));
        $currencies_module = $this->getCurrency((int)$cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
    * Alias für Configuration::get() mit Prefix
    * @param string key
    * @param int id_lang
    * @param int id_shop_group
    * @param int id_shop
    * @param string html
    * @return mixed
    */
    public function getConfig($key, $id_lang = null, $id_shop_group = null, $id_shop = null, $default = false)
    {
        return Configuration::get($this->cp.$key, $id_lang, $id_shop_group, $id_shop, $default);
    }

    public function getConfigGlobal($key, $idLang = null)
    {
        return Configuration::getGlobalValue($this->cp.$key, $idLang);
    }

    /**
    * Alias für Configuration::updateValue() mit Prefix
    * @param string key
    * @param mixed values
    * @param bool html
    * @param int id_shop_group
    * @param int id_shop
    * @return bool
    */
    public function setConfig($key, $values, $html = false, $id_shop_group = null, $id_shop = null)
    {
        return Configuration::updateValue($this->cp.$key, $values, $html, $id_shop_group, $id_shop);
    }

    public function setConfigGlobal($name, $value, $html = false)
    {
        return Configuration::updateGlobalValue($this->cp.$name, $value, $html);
    }

    /**
    * Alias für Configuration::deleteByName() mit Prefix
    * @param string key
    * @return bool
    */
    public function deleteConfig($key)
    {
        return Configuration::deleteByName($this->cp.$key);
    }

    /**
    * Alias für Tools::getValue mit Prefix
    * @param string key
    * @param string default_value
    * @return bool
    */
    public function getValue($key, $default_value = false)
    {
        return Tools::getValue($this->cp.$key, $default_value);
    }

    /**
    * Alias für Tools::isSubmit mit Prefix
    * @param string key
    * @param string default_value
    * @return bool
    */
    public function isSubmit($key, $default_value = false)
    {
        return Tools::isSubmit($this->cp.$key, $default_value);
    }

    /**
    * Ermittelt alle E-Mail Templates im ORdner /os_mail, die als Datei für dieses modul vorliegen
    * @param null
    * @return array
    */
    public function getOrderStatusMails()
    {
        $files = array();

        $this->gcHook(__FUNCTION__, 'pre', $files);

        if (is_dir($this->local_path.'os_mail')) {
            foreach (Language::getLanguages(true) as $language) {
                if (is_dir($this->local_path.'os_mail/'.$language['iso_code'].'/')) {
                    foreach (scandir($this->local_path.'os_mail/'.$language['iso_code'].'/') as $file) {
                        if ($file == $this->name.'.txt') {
                            $files[Tools::substr($file, 0, -4)][] = $language['iso_code'].'/'.$file;
                        }

                        if ($file == $this->name.'.html') {
                            $files[Tools::substr($file, 0, -4)][] = $language['iso_code'].'/'.$file;
                        }
                    }
                }
            }
        }

        $this->gcHook(__FUNCTION__, 'post', $files);

        return $files;
    }

    /**
    * Internes Hook System um Original GC Module ohne Änderungen beizubehalten
    * @param string method
    * @param string name
    * @param array mixed
    * @return mixed|null
    */
    private function gcHook($method, $name, &$params = null)
    {
        $method = ltrim($method, '_');
        $method = 'gc'.Tools::ucfirst($method).Tools::ucfirst($name);

        if (method_exists($this, $method)) {
            return $this->{$method}($params);
        } else {
            return null;
        }
    }

    public function gcTool($method, $params = null)
    {
        if (is_file(dirname(__FILE__).'/gc_tools.php')) {
            require_once(dirname(__FILE__).'/gc_tools.php');
            
            $gc_tools_name = __CLASS__.'_Tools';
            
            $gc_tools = new $gc_tools_name();
            $gc_tools->init($this);
            $return = $gc_tools->$method($params);
            
            $this->_errors = $gc_tools->_errors;

            return $return;
        } else {
            return true;
        }
    }

    public function smartyAssign($params)
    {
        if ($params) {
            $this->smarty->assign($params);
        }
    }
}
